# -*- coding: utf-8 -*-
"""
Created on Wed Aug 28 14:34:05 2019

@author: TS
"""

import os
import logging
import tornado.web
import tornado.websocket as websocket
import tornado.ioloop
import tornado.options
import time
import queue
import threading
import json
import uuid
from tornado.options import define, options
import mmap
import contextlib
import time


define("port", default=12306, help="run on the given port", type=int)
global content_1

class Application(tornado.web.Application):
    def __init__(self):
        handlers = [
            (r"/", First_Handler),
            (r"/ConnectRequest", Connect_Handler),
            (r"/HeartBeat", HeartBeat_Handler),
            (r"/TaskRequest", Task_Handler),
            (r"/Ws", MyWebSocketHandler),
        ]
        settings = dict(debug=True)
        tornado.web.Application.__init__(self, handlers, **settings)


###############################################################################
class First_Handler(tornado.web.RequestHandler):
    def get(self):
        print("get data from ts")
        self.write("RecvConnFromTS")
        
    def post(self):
        print(self.request.body.decode('utf-8'))
        self.write("RecvConnFromTS")
        MyWebSocketHandler.send_demand_updates(self.request.body.decode('utf-8'))
###############################################################################


class MyWebSocketHandler(websocket.WebSocketHandler):

    # 保存连接的用户，用于后续推送消息
    connect_users = set()

    def open(self):
        #print("WebSocket opened")
        # 打开连接时将用户保存到connect_users中
        self.connect_users.add(self)

    def on_message(self, message):
        print('收到的信息为：' + message)

    def on_close(self):
        #print("WebSocket closed")
        # 关闭连接时将用户从connect_users中移除
        self.connect_users.remove(self)

    def check_origin(self, origin):
        # 此处用于跨域访问
        return True
    
    
    @classmethod
    def send_demand_updates(cls, message):
        # 使用@classmethod可以使类方法在调用的时候不用进行实例化
        # 给所有用户推送消息（此处可以根据需要，修改为给指定用户进行推送消息）
        for user in cls.connect_users:
            user.write_message(message)


###########
# Connect #
################################################################################
class Connect_Handler(tornado.web.RequestHandler):
    def get(self):
        print(time.strftime("%H:%M:%S ", time.localtime()) + "收到TS连接请求")
        self.write("RecvConnFromTS")
###############################################################################


#############
# HeartBeat #
###############################################################################
class HeartBeat_Handler(tornado.web.RequestHandler):
    def get(self):
        print(time.strftime("%H:%M:%S ", time.localtime()) +"收到TS心跳请求")
        self.write("ServerIsAlive")
###############################################################################


##########
# Task #
###############################################################################
class Task_Handler(tornado.web.RequestHandler):
    content_1 = ""
    def get(self):
        #print(time.strftime("%H:%M:%S ", time.localtime())+ "收到TaskRequest")
        #这里将下单内容，查询内容，撤单内容等作为Response返回给TS客户端
        with contextlib.closing(mmap.mmap(-1, 1024, tagname='OrderSendQ', access=mmap.ACCESS_WRITE)) as m:
            OrderSendQ = str(m.read(1024).replace(b'\x00', b''), encoding = "utf-8")

        with contextlib.closing(mmap.mmap(-1, 1024, tagname='OrderCancelQ', access=mmap.ACCESS_WRITE)) as m:
            OrderCancelQ = str(m.read(1024).replace(b'\x00', b''), encoding = "utf-8")

        with contextlib.closing(mmap.mmap(-1, 1024, tagname='InfoQueryQ', access=mmap.ACCESS_WRITE)) as m:
            InfoQueryQ = str(m.read(1024).replace(b'\x00', b''), encoding = "utf-8")
            
        if len(OrderSendQ)>0 or len(OrderCancelQ)>0 or len(InfoQueryQ)>0:
            content=buildJsonFromString(OrderSendQ,OrderCancelQ,InfoQueryQ)

            self.write(content)
            self.flush()
            self.write(" ")
            self.flush()
            #s1 =  str(m.read(1024).replace(b'\x00', b''), encoding = "utf-8")
            #content_1 = content


def buildStringFromQueue(myQueue):
    result = ''
    while(not myQueue.empty()):
        item=myQueue.get(True, timeout=0.1)
        result += item+'&'
    return result

def buildJsonFromString(sendInfo, cancelInfo, queryInfo):
    dict ={}
    if len(sendInfo)>0:
        dict['DoSend'] = sendInfo
    if len(cancelInfo)>0:
        dict['DoCancel'] = cancelInfo
    if len(queryInfo)>0:
        dict['DoQuery'] = queryInfo
    result = json.dumps(dict)
    return result
###############################################################################


################
# start Server #
###############################################################
def startServer():
    tornado.options.parse_command_line()
    app = Application()
    app.listen(options.port)
    tornado.ioloop.IOLoop.instance().start()
    print(time.strftime("%H:%M:%S ", time.localtime()) + "server start now")
#################################################################


#########
#set Log#
########################################################################################################
def log_fun():
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    logpath = os.path.dirname(os.getcwd()) +"/"+time.strftime('%Y%m%d',time.localtime(time.time()))+'.log'
    fh = logging.FileHandler(logpath,mode='w')
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(logging.Formatter("%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s"))
    logger.addHandler(fh)

OrderSendQ = queue.Queue()  # 存储下单指令
OrderCancelQ = queue.Queue()  # 存储撤单指令
InfoQueryQ = queue.Queue()  # 存储查询指令

def main_fun():
    try:
        startServer()
    except Exception as ex:
        print("stop by : ", ex)

#######
if __name__ == "__main__":
    log_fun()
    main_fun()


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 28 14:34:05 2019

@author: TS
"""
import uuid
import queue
import time
import threading
import asyncio
import websockets
import sys                                                                  
import signal
import mmap
import contextlib
import time



#构建发单字符串
def buildSendInfo():
    Account = "410038111484"
    Id = str(uuid.uuid1())
    symbol = "600000.SH"
    Quantity = str(200)
    OrderAction = "Buy"
    Flag = "Limit"
    LimitPrice = str(10.09)
    LimitOrder = Flag+"|"+Account+"|"+symbol+"|"+Quantity+"|"+OrderAction+"|"+LimitPrice+"|"+Id
    _Account = "410038111484"
    _Id = str(uuid.uuid1())
    _Symbol = "000001.SZ"
    _Quantity = str(100)
    _OrderAction = "Sell"
    _Flag = "Market"
    MarketOrder = _Flag+"|"+_Account+"|"+_Symbol+"|"+_Quantity+"|"+_OrderAction+"|"+_Id
    content = LimitOrder #+ "&" + MarketOrder
    return content


#构建撤单字符串
def buildCancelInfo():
    Account = "410038111484"
    Id = str(uuid.uuid1())
    Flag = 'Cancel'
    CancelID = '你需要撤单的编号，在委托申报时由你唯一指定'
    content = Flag + "|" + Account + "|" + CancelID + "|" + Id
    return  content


#构建查询字符串
def buildQueryInfo():
    #
    Id1 = str(uuid.uuid1())
    AcctID = "410038111484"
    Flag1 = "Account"
    QueryAccount = Flag1 + "|" + AcctID + "|" + Id1 # 如果AccountID为空，返回所有
    #
    Id2 = str(uuid.uuid1())
    Acct2 = "410038111484"
    Symb2 = "000001.SZ"
    PositionID = "SL" + Acct2 + Symb2
    Flag2 = "Position"
    QueryPosition = Flag2 + "|" + PositionID + "|" + Id2 # 如果PositionID为空，返回所有
    #
    Id3 = str(uuid.uuid1())
    Flag3 = "Order"
    LocalId = "131234" # 已报委托的用户指定编号
    QueryOrder = Flag3 + "|" + LocalId + "|" + Id3 # 如果localID为空，返回所有
    #
    content = QueryAccount + "&" + QueryPosition+"&"+QueryOrder
    return content
##########################################################################################################


#下单函数，orders_str为下单内容
def DoSend(orders_str):
    with contextlib.closing(mmap.mmap(-1, 1024, tagname='OrderSendQ', access=mmap.ACCESS_WRITE)) as m:
        #这里写发单的内容
        # for i in range(1, 3):
        #     print(i)
        m.seek(0)
        temp_str = bytes(orders_str, encoding = "utf8")
        m.write(temp_str)
        m.flush()
        time.sleep(0.09999)
        m.write(b" ")
        m.flush()


def DoCancel(cancel_str):
    with contextlib.closing(mmap.mmap(-1, 1024, tagname='OrderCancelQ', access=mmap.ACCESS_WRITE)) as m:
        #这里写发单的内容
        # for i in range(1, 3):
        #     print(i)
        m.seek(0)
        temp_str = bytes(cancel_str, encoding = "utf8")
        m.write(temp_str)
        m.flush()
        time.sleep(0.09999)
        m.write(b" ")
        m.flush()


def DoQuery(query_str):
    with contextlib.closing(mmap.mmap(-1, 1024, tagname='InfoQueryQ', access=mmap.ACCESS_WRITE)) as m:
        #这里写发单的内容
        # for i in range(1, 3):
        #     print(i)
        m.seek(0)
        temp_str = bytes(query_str, encoding = "utf8")
        m.write(temp_str)
        m.flush()
        time.sleep(0.09999)
        m.write(b" ")
        m.flush()

response_value = ""

async def get_ws():
    uri = "ws://localhost:12306/Ws"
    async with websockets.connect(uri) as websocket:
        greeting = await websocket.recv()
        global response_value
        response_value = greeting

def response():
    asyncio.get_event_loop().run_until_complete(get_ws())
    return response_value


def quit(signum, frame):
    print('')
    print('stop fusion')
    sys.exit()


def get_response():
    signal.signal(signal.SIGINT, quit)                                
    signal.signal(signal.SIGTERM, quit)
    while(True):
        #这里获取成交回报
        response_value = response() 
        print("get message : ", response_value)
        return response_value

if __name__ == "__main__":

    DoSend(buildSendInfo())
    DoCancel(buildCancelInfo())
    DoQuery(buildQueryInfo())


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 28 14:34:05 2019

@author: TS
"""


from ts_response import response


if __name__ == "__main__":
	print("start getting response")
	while(True):
		try:
			response_value = response()
			print("response_value: ", response_value)
		except Exception as ex:
			print("错误: ", ex)


from setuptools import find_packages,setup


setup(
    name = 'tradingts', 
    version="0.0.3",
    author="ts_python_user",
    description="ts python package",
    url="https://github.com/",
    packages=find_packages(), 
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)